package database

import (
	"context"
	"os"
	"testing"

	"git.xeserv.us/xena/route/internal/routecrypto"
	"github.com/Xe/uuid"
)

const (
	cryptoKey    = `I7EzBRcoFtbW6RuqKdFjxYTHC_bpQLPJO3zyS2R-sf8=`
	certNameBase = "cert."
	certValue    = "hunter2"
)

func testCerts(ctx context.Context, t *testing.T, c Certs) {
	cn := certNameBase + uuid.New()

	t.Run("put", func(t *testing.T) {
		data := certValue + uuid.New()

		err := c.Put(ctx, cn, []byte(data))
		if err != nil {
			t.Fatal(err)
		}

		data2, err := c.Get(ctx, cn)
		if err != nil {
			t.Fatal(err)
		}

		if data != string(data2) {
			t.Fatalf("expected data to be %q, got %q", data, string(data2))
		}
	})

	t.Run("get", func(t *testing.T) {
		_, err := c.Get(ctx, cn)
		if err != nil {
			t.Fatal(err)
		}
	})

	t.Run("delete", func(t *testing.T) {
		err := c.Delete(ctx, cn)
		if err != nil {
			t.Fatal(err)
		}

		_, err = c.Get(ctx, cn)
		if err == nil {
			t.Fatal("able to fetch a deleted item")
		}
	})
}

func TestBoltDBCertStorage(t *testing.T) {
	k, err := routecrypto.ParseKey(cryptoKey)
	if err != nil {
		t.Fatal(err)
	}

	p := uuid.New()
	defer os.RemoveAll(p)

	st, err := NewBoltStorage(p, k)
	if err != nil {
		t.Fatal(err)
	}
	defer st.Close()

	ctx, cancel := context.WithCancel(context.Background())
	defer cancel()

	testCerts(ctx, t, st.Certs())
}
